package com.ashokit.saga.payment.service;

import com.ashokit.saga.commons.dto.OrderRequestDto;
import com.ashokit.saga.commons.dto.PaymentRequestDto;
import com.ashokit.saga.commons.event.OrderEvent;
import com.ashokit.saga.commons.event.PaymentEvent;
import com.ashokit.saga.commons.event.PaymentStatus;
import com.ashokit.saga.payment.entity.UserBalance;
import com.ashokit.saga.payment.entity.UserTransaction;
import com.ashokit.saga.payment.repository.UserBalanceRepository;
import com.ashokit.saga.payment.repository.UserTransactionRepository;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.PostConstruct;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service
public class PaymentService {

    @Autowired
    private UserBalanceRepository userBalanceRepository;
    @Autowired
    private UserTransactionRepository userTransactionRepository;

    @PostConstruct
    public void initUserBalanceInDB() {
        userBalanceRepository.saveAll(Stream.of(new UserBalance(101, 5000),
                new UserBalance(102, 3000),
                new UserBalance(103, 4200),
                new UserBalance(104, 20000),
                new UserBalance(105, 999)).collect(Collectors.toList()));
    }

    /**
     * // get the user id
     * // check the balance availability
     * // if balance sufficient -> Payment completed and deduct amount from DB
     * // if payment not sufficient -> cancel order event and update the amount in DB
     **/
    @Transactional
    public PaymentEvent newOrderEvent(OrderEvent orderEvent) {
    	
    	  // get the user id
        // check the balance availability
        // if balance sufficient -> Payment completed and deduct amount from DB
        // if payment not sufficient -> cancel order event and update the amount in DB
    
    	
        OrderRequestDto orderRequestDto = orderEvent.getOrderRequestDto();

        PaymentRequestDto paymentRequestDto = new PaymentRequestDto(orderRequestDto.getOrderId(),
                orderRequestDto.getUserId(), orderRequestDto.getAmount());

        return userBalanceRepository.findById(orderRequestDto.getUserId())   // get the user id
                .filter(ub -> ub.getPrice() > orderRequestDto.getAmount()) //Verifying the balance sufficient or not
                .map(ub -> {
                    ub.setPrice(ub.getPrice() - orderRequestDto.getAmount()); //Deduct the order amount in DB
                    userTransactionRepository.save(new UserTransaction(
                    		orderRequestDto.getOrderId(), 
                    		orderRequestDto.getUserId(), 
                    		orderRequestDto.getAmount())); //Saving UserTransaction Table
                    return new PaymentEvent(paymentRequestDto, PaymentStatus.PAYMENT_COMPLETED);
                }).orElse(new PaymentEvent(paymentRequestDto, PaymentStatus.PAYMENT_FAILED));

    }

    @Transactional
    public void cancelOrderEvent(OrderEvent orderEvent) {

        userTransactionRepository.findById(orderEvent.getOrderRequestDto().getOrderId())
                .ifPresent(ut->{
                    userTransactionRepository.delete(ut);
                    userTransactionRepository.findById(ut.getUserId())
                            .ifPresent(ub->ub.setAmount(ub.getAmount()+ut.getAmount()));
                });
    }
}
